@@ -13,6 +13,7 @@
  *
  *  The full text of the license is available at http://www.gnu.org/copyleft/lesser.html
  */
+
 #include <Rcpp.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -30,6 +31,7 @@
 //#include <malloc.h>
 //#endif
 //#endif
+
 #include "Framework.h"
 
 // //RankSwapping
@@ -41,13 +43,12 @@ enum{
 #include "RankSwapping.h"
 
 
- //Mdav
-enum{
-   es_NbHashBitXX  = 16,
-   es_NbHashListXX = 1 << es_NbHashBitXX
- };
-#include "Mdav.h"
-
+//Mdav
+//enum{
+//   es_NbHashBitXX  = 16,
+//   es_NbHashListXX = 1 << es_NbHashBitXX
+// };
+//#include "Mdav.h"
 
 //Suda2
 #include "Suda2.h"
@@ -56,9 +57,4 @@ enum{
 
 #include "Measure_Risk.h"
 #include "Measure_Hierarchical.h"
-
 #include "Measure_Threshold.h"
-
-#include "LocalRecProg/kneibmatch.cpp"
-
-
